/*
 * Copyright 2012-2017 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.vm.cpu.agent.internal;

import static com.redhat.thermostat.vm.cpu.agent.internal.VmCpuStatDAOImpl.ConfigurationCreator;
import static com.redhat.thermostat.vm.cpu.agent.internal.VmCpuStatDAOImpl.JsonHelper;
import static org.mockito.Matchers.anyListOf;
import static org.mockito.Matchers.eq;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import java.net.URI;
import java.util.Collections;

import com.redhat.thermostat.agent.http.HttpRequestService;
import com.redhat.thermostat.common.config.experimental.ConfigurationInfoSource;
import com.redhat.thermostat.common.plugin.PluginConfiguration;
import com.redhat.thermostat.common.plugin.SystemID;
import com.redhat.thermostat.vm.cpu.agent.model.VmCpuStat;
import org.junit.Before;
import org.junit.Test;

public class VmCpuStatDAOImplTest {

    private static final String AGENT_ID = "some-agent";
    private static final String JSON = "{\"this\":\"is\",\"also\":\"JSON\"}";
    private static final URI GATEWAY_URI = URI.create("http://example.com/jvm-cpu/");
    private static final String SYSTEM_ID = "1234";
    public static final String JVM_ID = "vm-123";

    private VmCpuStat stat;
    private JsonHelper jsonHelper;
    private VmCpuStatDAOImpl dao;

    private HttpRequestService httpRequestService;

    @Before
    public void setup() throws Exception {
        stat = new VmCpuStat();
        stat.setAgentId(AGENT_ID);
        stat.setTimeStamp(1234L);
        stat.setCpuLoad(42.0);
        stat.setVmId(JVM_ID);

        jsonHelper = mock(JsonHelper.class);
        when(jsonHelper.toJson(anyListOf(VmCpuStat.class))).thenReturn(JSON);

        ConfigurationInfoSource source = mock(ConfigurationInfoSource.class);
        PluginConfiguration config = mock(PluginConfiguration.class);
        when(config.getGatewayURL()).thenReturn(GATEWAY_URI);
        ConfigurationCreator creator = mock(ConfigurationCreator.class);
        when(creator.create(source)).thenReturn(config);
        SystemID systemID = mock(SystemID.class);
        when(systemID.getSystemID()).thenReturn(SYSTEM_ID);

        httpRequestService = mock(HttpRequestService.class);
        dao = new VmCpuStatDAOImpl(jsonHelper, creator);
        dao.bindHttpRequestService(httpRequestService);
        dao.bindConfigurationInfoSource(source);
        dao.bindSystemID(systemID);
    }

    @Test
    public void verifyAddVmCpuStat() throws Exception {
        dao.activate();
        dao.put(stat);
        verify(jsonHelper).toJson(eq(Collections.singletonList(stat)));
        verify(httpRequestService).sendHttpRequest(JSON, GATEWAY_URI.resolve("systems/" + SYSTEM_ID + "/jvms/" + JVM_ID), HttpRequestService.Method.POST);
    }
}
